// Copyright (c) 2017, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of expr;

/// Compute result of the given [substitution] in [input] at [index]. An
/// existing [mapping] can be specified. If a conflict arises this function
/// throws an error.
Expr substituteTransform(Expr input, int index, Transform substitution,
    [Mapping mapping]) {
  final m = mapping ?? new Mapping();
  final subExpr = getSubExpr(input, index);
  final t1 = new Transform(substitution.source, subExpr);
  mergeMappings(m, findSmallestTransforms(t1));
  if (isNormalizedMapping(m)) {
    final pMapping = getPatternMapping(m);

    // Flatten normalized mapping and check if all transforms can be
    // eliminated. We do not apply evaluation here as the substitution should
    // match directly.
    for (final from in m.keys) {
      final transform = new Transform(from, m[from].single);
      final reduced = mapTransform(transform, pMapping);
      if (!isIdentityTransform(reduced)) {
        throw new ArgumentError('non-trivial transforms in mapping');
      }
    }

    final replacement = mapExpr(substitution.target, pMapping);
    return replaceSubExpr(input, index, replacement);
  } else {
    throw new ArgumentError('conflicts in mapping normalization');
  }
}
