// Copyright (c) 2017, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of expr;

/// Transform between [source] and [target].
class Transform {
  final Expr source, target;

  Transform(this.source, this.target);

  @override
  int get hashCode => hash2(source, target);

  @override
  bool operator ==(other) {
    return other is Transform &&
        source == other.source &&
        target == other.target;
  }
}

/// Invert single transform.
Transform invertTransform(Transform t) {
  return new Transform(t.target, t.source);
}

/// Invert list of transforms (convert from-a-to-b into from-b-to-a)
List<Transform> invertTransforms(List<Transform> input) {
  return input.map(invertTransform).toList();
}

/// Evaluate single transform. Stores evaluation information in [evaluateSource]
/// and [evaluateTarget].
Transform tryEvaluateTransform(Transform t, Set<int> canEval, EvalExpr eval,
    [Set<int> evaluateSource, Set<int> evaluateTarget]) {
  final source = tryEvaluateExpr(t.source, canEval, eval, evaluateSource);
  final target = tryEvaluateExpr(t.target, canEval, eval, evaluateTarget);
  return new Transform(source, target);
}

/// Return true if the given transform [t] is an identity transform.
bool isIdentityTransform(Transform t) {
  return t.source == t.target;
}
