// Copyright (c) 2017, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

library expr.test.codec_test;

import 'package:test/test.dart';
import 'package:expr/expr.dart';
import 'package:expr/testenv.dart';

final testExpressions = [
  parse('a + b(a * c + 99.987654321 + 0.012 + 0.012)'),
  parse('1 + 1 + #a - #a + #b(ab * #b(1)) + 3 + 3'),
  tryEvaluateExpr(parse('f(-100)'), canEval, eval),
  tryEvaluateExpr(parse('f(-99.99, -99.99)'), canEval, eval),
];

void main() {
  test('encodeExprAsBytes', () {
    // Basic tests.
    for (final expr in testExpressions) {
      expect(decodeBytesIntoExpr(encodeExprAsBytes(expr)), equals(expr));
    }

    // Generate expression with more than 255 objects.
    final exprObjects = new List<ExprObject>();
    exprObjects.add(new ExprObject(ExprType.function, 1, 1000));
    for (var i = 0; i < 1000; i++) {
      exprObjects.add(new ExprObject(ExprType.number, i, 0));
    }

    // Test with codec.
    final expr = new Expr(exprObjects);
    expect(decodeBytesIntoExpr(encodeExprAsBytes(expr)), equals(expr));
  });
}
