// Copyright (c) 2017, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

library expr.test.misc_test;

import 'package:test/test.dart';
import 'package:expr/expr.dart';
import 'package:expr/testenv.dart';

void main() {
  test('detectRearrangementPattern', () {
    final t1 = parseTransform('#a#b => #c#b');
    final t2 = parseTransform('#a#b => #b#a');

    expect(detectRearrangementPattern(t1), equals([]));
    expect(detectRearrangementPattern(t2),
        equals([getId(ExprType.function, '*')]));
  });

  test('detectRedistributionPattern', () {
    int id(String lbl) => getId(ExprType.function, lbl);
    final key = new DistFnKey(id('*'), id('+'));

    final fail = [
      parseTransform('f(g(#x, #y, #z)) => g(f(#x), f(#y), f(#z))'),
      parseTransform('f(g(#x, #y)) => g(f(#x), h(#y))'),
      parseTransform('f(h(#x, #y)) => g(f(#x), f(#y))'),
      parseTransform('f(g(#x, y)) => g(f(#x), f(#y))'),
      parseTransform('#a*(#b + #c) => #d*#b + #a#c'),
      parseTransform('#a*(#b + #c) => #a*#d + #a#c'),
      parseTransform('(#b + #c)*#a => #a#b + #a#c')
    ];
    final pass = {
      parseTransform('#a*(#b + #c) => #a#b + #a#c'): [1],
      parseTransform('(#b + #c)*#a => #b#a + #c#a'): [0]
    };

    for (final t in fail) {
      expect(detectRedistributionPattern(t), equals({}));
    }
    for (final t in pass.keys) {
      expect(detectRedistributionPattern(t), equals({key: pass[t]}));
    }
  });
}
