// Copyright (c) 2017, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

library expr.test.parse_test;

import 'package:test/test.dart';
import 'package:expr/testenv.dart';

void main() {
  final a = symbol('a');
  final b = symbol('b');
  final c = symbol('c');
  final p = pattern('p');
  final arrow = function('=>');
  final add = function('+');
  final sub = function('-');
  final mul = function('*');
  final pow = function('^');
  final neg = function('~');
  final fun = function('fun');

  test('parseExpr', () {
    final tests = {
      'a + b --> c': arrow(add(a, b), c),
      'a + b => c': arrow(add(a, b), c),
      'a + b': add(a, b),
      'a() + b': add(a, b),
      'a + b() ': add(a, b),
      '#p + b': add(p(), b),
      'a * b + c': add(mul(a, b), c),
      'a^2b + c': add(pow(a, mul(n(2), b)), c),
      '(a^2)b + c': add(mul(pow(a, n(2)), b), c),
      '(a^.05)b + c': add(mul(pow(a, n(.05)), b), c),
      'fun(0., a b)': fun(n(0), mul(a, b)),
      '(2 + -4)': add(n(2), neg(n(4))),
      '(2 + 4)*(10 - 4)*(2 * 3)':
          mul(mul(add(n(2), n(4)), sub(n(10), n(4))), mul(n(2), n(3)))
    };

    for (final input in tests.keys) {
      final output = parse(input);
      expect(output, equals(tests[input]));
    }
  });
}
