// Copyright (c) 2017, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

library expr.test.testenv_test;

import 'package:test/test.dart';
import 'package:expr/expr.dart';
import 'package:expr/testenv.dart';

import 'misc/matchers.dart';

final easyPrintTests = {
  'a (b c)': 'a(bc)',
  'a * 1': 'a(1)',
  'a * -1': 'a(¬1)',
  '#a + b(c*d, #ee*f)': '#a + b(cd, (#ee)f)'
};

void main() {
  test('easyPrintExpr', () {
    loadTemplates(defaultTemplates);
    loadTemplates({'*': r'${.0(+):}${:1(*).}'});

    easyPrintTests.forEach((input, output) {
      expect(easyPrintExpr(parse(input)), output);
    });

    expect(easyPrintExpr(tryEvaluateExpr(parse('a * -1'), canEval, eval)),
        equals('a(-1)'));

    expect(() => easyPrintExpr(new Expr([new ExprObject(null, 0, 0)])),
        thrws('unknown object type'));
  });
}
